@@ -16,6 +16,8 @@ __email__ = "shyue@mit.edu"
 __date__ = "May 2, 2012"
 
 import logging
+import subprocess
+import time
 from abc import ABCMeta, abstractmethod, abstractproperty
 import json
 
@@ -27,13 +29,42 @@ class Custodian(object):
 
     1. Let's say you have defined a list of jobs as [job1, job2, job3, ...] and
        you have defined a list of possible error handlers as [err1, err2, ...]
-    2. Custodian will run the jobs in the order of job1, job2, ...
+    2. Custodian will run the jobs in the order of job1, job2, ... During each
+       job, custodian will monitor for errors using the handlers that have
+       is_monitor == True. If an error is detected, corrective measures are
+       taken and the particular job is rerun.
     3. At the end of each individual job, Custodian will run through the list
-       error handlers. If an error is detected, corrective measures are taken
-       and the particular job is rerun.
+       error handlers that have is_monitor == False. If an error is detected,
+       corrective measures are taken and the particular job is rerun.
+
+    .. attribute: max_errors
+
+        Maximum number of errors allowed.
+
+    .. attribute: handlers
+
+        Error handlers that are not Monitors.
+
+    .. attribute: monitors
+
+        Error handlers that are Monitors, i.e., handlers that monitors a job
+        as it is being run.
+
+    .. attribute: polling_time_step
+
+        The length of time in seconds between steps in which a job is
+        checked for completion.
+
+    .. attribute: monitor_freq
+
+        The number of polling steps before monitoring occurs. For example,
+        if you have a polling_time_step of 10seconds and a monitor_freq of
+        30, this means that Custodian uses the monitors to check for errors
+        every 30 x 10 = 300 seconds, i.e., 5 minutes.
     """
 
-    def __init__(self, handlers, jobs, max_errors=1):
+    def __init__(self, handlers, jobs, max_errors=1, polling_time_step=10,
+                 monitor_freq=30):
         """
         Args:
             handlers:
@@ -44,10 +75,22 @@ class Custodian(object):
                 double-relaxation.
             max_errors:
                 Maximum number of errors allowed before exiting.
+            polling_time_step:
+                The length of time in seconds between steps in which a
+                job is checked for completion. Defaults to 10 seconds.
+            monitor_freq:
+                The number of polling steps before monitoring occurs. For
+                example, if you have a polling_time_step of 10seconds and a
+                monitor_freq of 30, this means that Custodian uses the
+                monitors to check for errors every 30 x 10 = 300 seconds,
+                i.e., 5 minutes.
         """
         self.max_errors = max_errors
         self.jobs = jobs
-        self.handlers = handlers
+        self.handlers = filter(lambda x: not x.is_monitor, handlers)
+        self.monitors = filter(lambda x: x.is_monitor, handlers)
+        self.polling_time_step = polling_time_step
+        self.monitor_freq = monitor_freq
 
     def run(self):
         """
@@ -64,27 +107,50 @@ class Custodian(object):
             for attempt in xrange(self.max_errors):
                 logging.info(
                     "Starting job no. {} ({}) attempt no. {}. Errors thus far"
-                    " = {}.".format(i + 1, job.name, attempt + 1,
-                                    total_errors))
+                    " = {}.".format(i + 1, job.name, attempt + 1, total_errors))
 
                 # If this is the start of the job, do the setup.
                 if not run_log[-1]["corrections"]:
                     job.setup()
 
-                # Run the job.
-                job.run()
-
+                p = job.run()
                 # Check for errors using the error handlers and perform
                 # corrections.
                 error = False
-                for h in self.handlers:
-                    if h.check():
-                        total_errors += 1
-                        d = h.correct()
-                        logging.error(str(d))
-                        run_log[-1]["corrections"].append(d)
-                        error = True
-                        break
+
+                # While the job is running, we use the handlers that are
+                # monitors to monitor the job.
+                if isinstance(p, subprocess.Popen):
+                    if self.monitors:
+                        n = 0
+                        while True:
+                            n += 1
+                            time.sleep(self.polling_time_step)
+                            if p.poll() is not None:
+                                break
+                            if n % self.monitor_freq == 0:
+                                for h in self.monitors:
+                                    if h.check():
+                                        p.terminate()
+                                        d = h.correct()
+                                        logging.error(str(d))
+                                        run_log[-1]["corrections"].append(d)
+                                        error = True
+                                        break
+                    else:
+                        p.wait()
+
+                # If there are no errors *during* the run, we now check for
+                # errors *after* the run using handlers that are not monitors.
+                if not error:
+                    for h in self.handlers:
+                        if h.check():
+                            total_errors += 1
+                            d = h.correct()
+                            logging.error(str(d))
+                            run_log[-1]["corrections"].append(d)
+                            error = True
+                            break
 
                 #Log the corrections to a json file.
                 with open("custodian.json", "w") as f:
@@ -132,6 +198,18 @@ class ErrorHandler(object):
         """
         pass
 
+    @abstractproperty
+    def is_monitor(self):
+        """
+        This property indicates whether the error handler is a monitor,
+        i.e., a handler that monitors a job as it is running. If a
+        monitor-type handler notices an error, the job will be sent a
+        termination signal, the error is then corrected,
+        and then the job is restarted. This is useful for catching errors
+        that occur early in the run but do not cause immediate failure.
+        """
+        return False
+
     @abstractproperty
     def to_dict(self):
         """
@@ -169,7 +247,8 @@ class Job(object):
     @abstractmethod
     def run(self):
         """
-        This method perform the actual work for the job.
+        This method perform the actual work for the job. If parallel error
+        checking is desired, this must return a Popen process.
         """
         pass
 
@@ -206,4 +285,4 @@ class Job(object):
         implemented until Python 3+.
         """
         raise NotImplementedError("Job objects must implement a from_dict"
-                                  "static method.")
\ No newline at end of file
+                                  "static method.")
