@@ -73,6 +73,10 @@ class ExampleHandler(ErrorHandler):
         print "Total < 50. Incrementing initial to {}".format(initial)
         return {"errors": "total < 50", "actions": "increment by 1"}
 
+    @property
+    def is_monitor(self):
+        return False
+
     @property
     def to_dict(self):
         return {}
